/** 	The class <tt>OvalObject</tt> creates a circular 
	polygon object which is drawn in @see <tt>RobotPanel</tt>.  
	It extends the abstract class @see <tt>ObjectClass</tt>.

	@author Carey Fry
	@version 1.0  */

import javax.swing.*;
import java.awt.*;

public class OvalObject extends ObjectClass {

private static final int NUMBER_OF_DIRECTIONS = 16;  // number of points on circle			
private double[] angle = new double[NUMBER_OF_DIRECTIONS];  // angles to find points

/** 	The method <tt>OvalObject> with no parameters is a
	constructor that calls the no-parameter constructor
	of the superclass <tt>ObjectClass</tt>.
	  */
 
public OvalObject() {
	super();
	} // END constructor

/** 	The method <tt>OvalObject> is a constructor that 
	calls the constructor of the superclass <tt>ObjectClass</tt>,
	which creates an instance of <tt>OvalObject</tt> with
	the size and color specified in the parameters.
	
	@param 	size	size of object
	@param  color	color of object
	*/

public OvalObject(String size, String color) {
	super(size, color);

	double ovalRadius = width/2;		// radius of oval
	double ovalCenterX = X + ovalRadius;	// X coordinate of center
	double ovalCenterY = Y + height/2;	// Y coordinate of center
	int ovalX, ovalY;			// X, Y coordinates of 
						// bounding rectangle

	// find points on oval to add to polygon
	for (int i = 0; i < NUMBER_OF_DIRECTIONS; i++) {
		angle[i] = ((2 * Math.PI * i) / NUMBER_OF_DIRECTIONS);
		ovalX = (int)(ovalCenterX + (ovalRadius * Math.cos(angle[i])));
		ovalY = (int)(ovalCenterY + (ovalRadius * Math.sin(angle[i])));
		p.addPoint(ovalX, ovalY);
	}// END for
}// END constructor

} // END class