/** Robot class for Assignment P

    @author Dan Kelly

		@version 1.0

*/



import java.lang.*;

import java.awt.*;



public class Robot {



  // Variables to hold the center of the robot, initialized

	private double centerOfRobotX = 100, centerOfRobotY = 100;

  // Temporary Variables to hold polygon points

	private double robotPolygonX, robotPolygonY,sensorPolygonX,sensorPolygonY;

	// Variables to hold the front of the robot polygon

	private double frontOfRobotX, frontOfRobotY;  

	// Array to hold angles

	private double[] angle = new double[NUMBER_OF_DIRECTIONS];

  // Declare angle to compare with the array

	private double angle1 = 0; 

  // Declare a new polygons for the robot and its sensor

	private Polygon robotPolygon=new Polygon();  

	private Polygon sensorPolygon=new Polygon();  

  // The radius of the robot, initialized

	private int robotRadius = 13;  

	// The radius of the sensor, initialized

	private int sensorRadius = 40;  

  // Constant for the number of directions

	private static final int NUMBER_OF_DIRECTIONS = 16; 



  /** Creates the Robot polygon and the polygon of its sensor */

	public Robot() {

    // Loop to create array of angle values for NUMBER_OF_DIRECTIONS

		for(int i=0;i<NUMBER_OF_DIRECTIONS;i++) {  

    	angle[i] = ((2*Math.PI*i)/NUMBER_OF_DIRECTIONS);

    }



    // Set the points of the robot and sensor polygons

	  setPolygonPoints();  



	}  // END Constructor



	/** Moves the robot

      @param mX Slope of the line that the robot is moving when the robot is controled by the X variable 

			@param mY Slope of the line that the robot is moving when the robot is controled by the Y variable 

			@param dx The distance moved along the X-Axis if controled by the X variable

			@param dy The distance moved along the Y-Axis if controled by the Y variable

			@param b <t>true</t> if controled by the X variable, <t>false</t> if controled by the Y variable 

			*/

	public void move (double mX, double mY, int dx, int dy, boolean b) {



         	// Move the robot

         	if(b) {

			centerOfRobotX += dx;

		  	centerOfRobotY += mX;

                  	frontOfRobotX += dx;

		  	frontOfRobotY += mX;		

		}

		else {

			centerOfRobotX += (1/mY);

		  	centerOfRobotY += dy;

		  	frontOfRobotX += (1/mY);

		  	frontOfRobotY += dy;

		}



		robotPolygon=new Polygon();

		sensorPolygon=new Polygon();

		setPolygonPoints();



	}  // END move

	

	/** Returns the robot polygon

      @return The robot polygon

			*/

	public Polygon getRobotPolygon() {

 		return robotPolygon; 

	}  // END getRobotPolygon



	/** Returns the sensor polygon

      @return The sensor polygon

			*/

	public Polygon getSensorPolygon() {

 		return sensorPolygon; 

	}  // END getSensorPolygon



	/** Checks if the bounding rectangle intersects with the sensor polygon

      @param p The polygon to be checked if it intersects with the sensor polygon

			@return <t>true</t> if the polygon is intersects the sensor polygon, <t>false</t> otherwise

			*/

	public boolean inSensorRange (Polygon p) {

	  Rectangle boundingRect = p.getBounds();

		// Check if a point is in the sensor range

		if (sensorPolygon.intersects(boundingRect.x,boundingRect.y,boundingRect.width,boundingRect.height))

			return true;

		else

			return false;

	}  // END inSensorRange



	/** Returns the X-variable of the front of the robot

      @return The X-variable of the front of the robot

			*/

	public int getFrontOfRobotX() {

		return (int)frontOfRobotX;

	}  // END getFrontOfRobotX



  /** Returns the Y-variable of the front of the robot

      @return The Y-variable of the front of the robot

			*/

	public int getFrontOfRobotY() {

		return (int)frontOfRobotY;

	}  // END getFrontOfRobotY



	/** Returns the X-variable of the center of the robot

      @return The X-variable of the center of the robot

			*/

	public int getCenterOfRobotX() {

  		return (int)centerOfRobotX;

	}  // END getCenterOfRobotX



	/** Returns the Y-variable of the center of the robot

      @return The Y-variable of the center of the robot

			*/

	public int getCenterOfRobotY() {

		return (int)centerOfRobotY;

	}  // END getCenterOfRobotY



	/** Sets the heading and the front of the robot */

	public void setHeading(int x, int y) {



    // Declare and calculate necessary variables to determine the correct angle

		double cangle = Math.acos(x/(Math.sqrt((x*x)+(y*y))));

		double sangle = Math.asin(y/(Math.sqrt((x*x)+(y*y))));



		if((cangle<=(Math.PI/2)) && (sangle>=0))      // First Quadrant

			angle1 = cangle;

		else if((cangle>(Math.PI/2)) && (sangle>=0))  // Second Quadrant

			angle1 = cangle;

		else if((cangle>(Math.PI/2)) && (sangle<0))   // Third Quadrant

			angle1 = ((2*Math.PI) - cangle);

		else  					      // Fourth Quadrant

			angle1 = ((2*Math.PI) + sangle);



    //Calculate the front of the robot

		setFront();  



	}  // END setHeading



  /** Changes the radius of the robot to <t>rr</t>

	    @param rr The new radius of the robot

			*/

	public void setRobotRadius(int rr) {

		// Create a new robot polygon with the new radius

		robotRadius = rr;

		robotPolygon=new Polygon();

		sensorPolygon=new Polygon();

		setPolygonPoints();

		setFront();

	}



  /** Changes the radius of the sensor to <t>sr</t>

	    @param sr The new sensor of the robot

			*/

	public void setSensorRadius(int sr) {

		// Create a new sensor polygon with the new radius

		sensorRadius = sr;

		robotPolygon=new Polygon();

		sensorPolygon=new Polygon();

		setPolygonPoints();

	}

	

	// Sets the points of the robot and sensor polygons	

	private void setPolygonPoints() {

    for(int i=0;i<NUMBER_OF_DIRECTIONS;i++) { // Loop for NUMBER_OF_DIRECTIONS 

      calculateCartesian(i);  // Call private method to calculate cartesian coordinates

      robotPolygon.addPoint((int)robotPolygonX,(int)robotPolygonY);  // Add point to the robot polygon

		  sensorPolygon.addPoint((int)sensorPolygonX,(int)sensorPolygonY);  // Add point to the sensor polygon

    } // END for



	}  // END setPolygonPoints



  // Calculate the cartesian coordinates of the sobot and sensor polygons

	private void calculateCartesian(int i) {

	  // Calculate the points for the robot polygon  

  	robotPolygonX = (centerOfRobotX + (robotRadius * Math.cos(angle[i])));

    robotPolygonY = (centerOfRobotY + (robotRadius * Math.sin(angle[i])));

		// Calculate the points for the sensor polygon 

		sensorPolygonX = (centerOfRobotX + (sensorRadius * Math.cos(angle[i])));

    sensorPolygonY = (centerOfRobotY + (sensorRadius * Math.sin(angle[i])));

	}  // END calculateCartesian



	private void setFront() {

	  // Set the front of the robot and the sensor range

		frontOfRobotX = (centerOfRobotX + (robotRadius * Math.cos(angle1)));

		frontOfRobotY = (centerOfRobotY + (robotRadius * Math.sin(-angle1)));

	}  // END setFront

}  // END Robot 

